/* Correlation Statistics */

options results
if ~show('P','TCALC') then do
	address command 'run turbocalc:turbocalc'
	address command 'waitforport TCALC'
	loadflag=1
end
address 'TCALC'
'DEFPUBSCREEN()'
/* Add-in Rexx Math Library needed for some routines */
signal on syntax
if ~show('l','rexxmathlib.library') then
   call addlib('rexxmathlib.library',0,-30) 
if ~show('l','rexxreqtools.library') then
   call addlib('rexxreqtools.library',0,-30)
if ~show('l','rexxsupport.library') then
   call addlib('rexxsupport.library',0,-30)
 /* add to library list */
signal off syntax

/* Start Main Routine */
if loadflag=1 then 'Load()'
'ActivateWindow()'
range=rtgetstring(,"Enter Cell Range for Input","Input Request") /*,,'rt_pubscrname="TCALC"')*/
colon=pos(":",range)
if colon=0 then do
	'Message "Please select a range before executing this script"'
	'DEFPUBSCREEN("Workbench")'
	exit
end

/* Find cell references and cell, column numbers */
start_cell=substr(range,1,colon-1)
end_cell=substr(range,colon+1)
start_row=cellrow(start_cell)
end_row=cellrow(end_cell)
start_col=cellcol(start_cell)
end_col=cellcol(end_cell)
NRows=end_row-start_row+1
NCols=end_col-start_col+1
if NCols>2 then do
	'Message "Only two columns allowed!"'
	'DEFPUBSCREEN("Workbench")'
	exit
end

/* Get cell reference for output range */
out_cell=rtgetstring(,"Enter Cell Reference for Output","Input Request") /*,,'rt_pubscrname="TCALC"')*/
if out_cell="" then do
	'DEFPUBSCREEN("Workbench")'
	exit
end
if length(out_cell)<2 | datatype(left(out_cell,1),'n')=1 then do
	'Message "Invalid cell reference"'
	'DEFPUBSCREEN("Workbench")'
	exit
end
/* Suppress Screen Redraw to Speed Things Up */
'Refresh 0'

/* Open a small output window on tcalc screen*/
fo=0
CR='0a'x
DisplayMsg="Calculating...Please Wait."||CR||"User input is disabled during calculation."||CR
if open(6Info, 'con:100/0/450/80/Progress/SCREEN TCALC', w) then do
	 call writeln(6Info, DisplayMsg)
	fo=1
end
else do
	'Message "TCALC Screen not available for Progress messages"'
end
CALL DELAY(150)

/* Get cell references for top cell in each column */
'SelectCell' start_cell
do col=start_col to end_col
	'GetCursorPos'
	top_cell.col=result
	'Column 1'
end
if fo then call writech(6Info,"Progress...0 ")
/* Get labels for later use on output */
'SelectCell' start_cell
'GetValue'
testlabel=result
testlabel=strip(testlabel)
if datatype(testlabel,'n')=1 then do
	labelflag=0
	do x=1 to NCols
	title.x="Column "||x
	end
end
else do
	labelflag=1
	NRows=NRows-1
	do x=1 to NCols
	'GetValue'
	str=result
	title.x=translate(strip(str),"_"," ")
	'Column 1'
	end
end
if fo then call writech(6Info,"10 ")

/* Get data from cell range */
col=start_col
lav=0
tot=0
count.=0
total.=0
do x=1 to NCols
	'SelectCell' top_cell.col
	if labelflag=1 then 'CursorDown 1'
	do y=1 to NRows
		'GetValue'
		valtest=result
		if datatype(valtest)='NUM' then do
			'GetValue'
			val=result
			data.x.y=val
			tot=tot+val
			total.x=tot
			count.x=1+count.x
		end
		'CursorDown 1'
	end
	col=col+1
	tot=0
	lav=0
	val=0
end
if fo then call writech(6Info,"20 ")
if count.1 ~= count.2 then do
	'Message "Equal sample sizes required - aborting!"'
	'DEFPUBSCREEN("Workbench")'
	exit
end

/* Calculate Means */
mean.=0
do x=1 to NCols
	mean.x=total.x/count.x
end
if fo then call writech(6Info,"30 ")

/* Calculate Standard deviation and Variance */
dat=0
meenx=0
tot.=0
Vsq.=0
diff.=0 /* Array holding difference between value and column mean */
sum.=0 /* Array holding sum of value minus mean of value squared */
sd.=0 /* Standard deviation array */
var.=0 /* Variance array */
do x=1 to NCols
	sum.x=0
	meenx=mean.x
	do y =1 to Nrows
		dat=data.x.y
		diff.x.y=dat-meenx
		sum.x=(dat-meenx)**2+(sum.x)
		tot.x=tot.x+data.x.y
		Vsq.x=Vsq.x+(data.x.y)**2
	end
	N=(count.x)-1
	var.x=(sum.x)/N
	sd.x=sqrt(var.x)
end
if fo then call writech(6Info,"40 ")
N=NRows
xy=0
summulti=0 /* Sum of product of diffs */
diffMult.=0 /* Array holding values of diffx times diffy */
do y=1 to N
	diffMult.y=(diff.1.y)*(diff.2.y)
	summulti=summulti+(diffMult.y)
	xy=xy+(data.1.y)*(data.2.y)
end

if fo then call writech(6Info,"50 ")


/* Calculate Pearson r */
rs=0
pr=0
pr=summulti/sqrt((sum.1)*(sum.2))
rs=((summulti)**2)/((sum.1)*(sum.2))
pr=sqrt(rs)
/*top=(N*xy)-(tot.1)*(tot.2)
bot=sqrt((N*(Vsq.1)-(tot.1)**2)*(N*(Vsq.2)-(tot.2)**2))
pr=top/bot
rs=pr**2*/

/* Calculate Confidence Interval for Pearson R */

Zr=.5*LOG(1+pr)-.5*LOG(1-pr)
JJ=sqrt(1/(n-3))
CIP95=Zr+1.96*JJ
CIN95=Zr-1.96*JJ
CIP99=Zr+2.58*JJ
CIN99=Zr-2.58*JJ
if fo then call writech(6Info,"60 ")

/* Test Against True Population Correlation */

PCRR=rtgetstring("0","Enter True Population Correlation (Optional)","Input Request")
PCORR=strip(PCRR)
if PCORR>.99 then PCORR=.99
if rtresult~=0 & PCORR~=0 then Do
	Zp=.5*LOG(1+PCORR)-.5*LOG(1-PCORR)
	Zed=(Zr-Zp)/JJ
	Call NPROB(zed)
End

/* Calculate standard Error */
sterr=0
t=0
df=N-2 /* Degrees of Freedom */
sterr=sqrt((1-rs)/df)
t=pr/sterr

/* Calculate T distribution function */

P=PROBT(t,df)
if t>=0 then P=1-P
PT=P*2
/* Calculate T Critical */

TCRIT1=TCRITICAL(.95,df)
TCRIT2=TCRITICAL(.99,df)
TCRIT3=TCRITICAL(.975,df)
if fo then call writech(6Info,"70 ")
TCRIT4=TCRITICAL(.995,df)

/* Calculate Column Squares and Products */
SQ.=0
PR.=0
TSQ.=0
TPR.=0
do x=1 to NCols
	do y=1 to NRows
		SQ.x=(data.x.y)**2
		TSQ.x=(TSQ.x)+(SQ.x)
	end
end
if fo then call writech(6Info,"80 ")
do x=1 to NCols
	do z=1 to NCols
		do y=1 to NRows
			PR.x.z=(data.x.y)*(data.z.y)
			TPR.x.z=(TPR.x.z)+(PR.x.z)
		end
	end
end
N=count.1
if fo then call writech(6Info,"90 ")
/* Calculate Correlation */
top.=0
bot.=0
corr.=0
do x =1 to NCols
	do z=1 to NCols
	top.x.z=(N*(TPR.x.z))-((total.x)*(total.z))
	bot.x.z=sqrt((N*(TSQ.x)-(total.x)**2)*(N*(TSQ.z)-(total.z)**2))
	corr.x.z=(top.x.z)/(bot.x.z)
	end
end
if fo then do
	call writeln(6Info,"100 ")
	call writeln(6Info,"Writing output to window...")
end

/* Output */
'SelectCell' out_cell
'ColumnWidth' 15
'Put "Correlation: UnGrouped Data"'
'CursorDown 2'
'Column 1'
do x=1 to NCols
	'GetCursorPos'
	first_cell.x=result
	title=""""||title.x||""""
	'Alignment 2'
	'Put' title
	'Column 1'
end
'SelectCell' first_cell.1
'Column -1'
'CursorDown 1'
do x=1 to NCols
	title=""""||title.x||""""
	'Alignment 2'
	'Put' title
	'CursorDown 1'
end
d=0
b=0
do x=1 to NCols
'SelectCell' first_cell.x
'CursorDown 1'
	do z=1 to NCols
		'Put' format(corr.x.z,,4)
		'CursorDown 1'
	end
end
'CursorDown 2'
'Column -1'
'Put' format(pr,,4)
'Column -1'
'Put "Pearson r:"'
'CursorDown 1'
'Put "r sq.:"'
'Column 1'
'Put' format(rs,,4)
'CursorDown 1'
'Put' format(Sterr,,4)
'Column -1'
'Put "Std. Error of r:"'
'CursorDown 1'
'Put "N:"'
'Column 1'
'Put' NRows
'CursorDown 1'
'Put' df
'Column -1'
'Put "df:"'
'CursorDown 1'
'Put "t:"'
'Column 1'
'Put' format(t,,6)
'CursorDown 1'
'Put' format(P,,6)
'Column -1'
'Put "P(T<=t) one-tail:"'
'CursorDown 1'
'Put "T-Critical (95%):"'
'CursorDown 1'
'Put "T-Critical (99%):"'
'CursorDown 1'
'Put "P(T<=t) two-tail:"'
'CursorDown 1'
'Put "T-Critical (95%):"'
'CursorDown 1'
'Put "T-Critical (99%):"'
'CursorUp 4'
'Column 1'
'Put' format(TCRIT1,,4)
'CursorDown 1'
'Put' format(TCRIT2,,4)
'CursorDown 1'
'Put' format(PT,,6)
'CursorDown 1'
'Put' format(TCRIT3,,4)
'CursorDown 1'
'Put' format(TCRIT4,,4)
'Column -1'
'CursorDown 1'
If PCORR~=0 then DO
'Put "Test Against True Correlation:"'
'CursorDown 1'
'Put "True Correlation:"'
'Column 1'
'Put' PCORR
'Column -1'
'CursorDown 1'
'Put "z:"'
'CursorDown 1'
'Put "Prob (z to Left tail):"'
'CursorDown 1'
'Put "Prob (z to Right tail):"'
'CursorDown 1'
'Put "Density Function:"'
'CursorUp 3'
'Column 1'
'Put' format(zed,,4)
'CursorDown 1'
'Put' format(PZ,,8)
'CursorDown 1'
'Put' format(Q,,8)
'CursorDown 1'
'Put' format(PDF,,6)
'Column -1'
'CursorDown 1'
'Put "Z Critical One-tail(95%):"'
'Column 1'
'Put' 1.65
'CursorDown 1'
'Put' 2.33
'Column -1'
'Put "Z Critical One-tail(99%):"'
'CursorDown 1'
'Put "Z Critical Two-tail(95%):"'
'Column 1'
'Put' 1.96
'CursorDown 1'
'Put' 2.58
'Column -1'
'Put "Z Critical Two-tail(99%):"'
'CursorDown 1'
End
'Put "Confidence Intervals"'
'CursorDown 1'
'Put "95% (+1.96):"'
'CursorDown 1'
'Put "95% (-1.96):"'
'CursorDown 1'
'Put "99% (+2.58):"'
'CursorDown 1'
'Put "99% (-2.58):"'
'CursorUp 3'
'Column 1'
'Put' format(CIP95,,4)
'CursorDown 1'
'Put' format(CIN95,,4)
'CursorDown 1'
'Put' format(CIP99,,4)
'CursorDown 1'
'Put' format(CIN99,,4)
'Refresh 1'
'Refresh 2'
/*'Message' "Finished"*/
/*indicate the main script is finished*/
DisplayMsg="Cleaning up ...."||CR||"Exiting"
result=writeln(6Info, DisplayMsg)
if result~=0 then do
	/*Wait 3 seconds*/
	CALL DELAY(150)
	/* close window*/
	result=close(6Info)
end
'DEFPUBSCREEN("Workbench")'
exit

/* Procedures */

cellrow: procedure
do
	parse arg cell
	do charpos=2 to length(cell)
	if datatype(substr(cell,charpos,1),n) then return substr(cell,charpos)
	end
	return 0
end
Return

cellcol: procedure
do
	parse arg cell
	labels="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	cell=upper(cell)
	len=length(cell)
	val=0
do charpos=1 to len
	if datatype(substr(cell,charpos,1),n) then
	do cell=reverse(substr(cell,1,charpos-1))
	do x=1 to length(cell)
	val=(26**(x-1))*pos(substr(cell,x,1),labels)+val
	end
	return val
	end
	end
	return 0
end
Return
/* It is important to put the exposed array at the end of the next line */
Sort: procedure expose NCols NRows data.
do x=1 to NCols
L=(xtoy(2,int(log(NRows-1)/log(2))))-1
	Do Until L<1
	L=trunc(int(L/2))
	Do J=1 to L
			Do K=J+L To NRows-1 By L
			I=K
			dumdat=data.x.I
			Do while I>L
				y=I-L
				If data.x.y ~> dumdat then Leave
				data.x.I=data.x.y
				I=I-L
			End
			data.x.I=dumdat
			End
		End
	End
End
Return

syntax:
	 if arg(1)='FAIL' then do
	    'Message "Library is unavailable."'
		'DEFPUBSCREEN("Workbench")'
	    exit
	    end
	'DEFPUBSCREEN("Workbench")'
	exit

Format:  procedure

	 arg number, before, after
	 CallLine = SIGL
	 if ~datatype(CallLine, 'N') then CallLine = '??'

	 /* Make sure we have a number as first (required) argument    */
	 if ~datatype(number, 'N') then do
	    if number = '' then
	       fc = 17     /* Wrong number of arguments 		  */
	    else
	       fc = 47     /* Arithmetic conversion error       	  */
	    signal FormatSyntaxError
	 end
	 num = number + 0
	 if before = '' & after = '' then
	    return num
	 else do
	    parse var num integer '.' fraction
	    if before = '' then before = length(integer)
	    if after = '' then after = length(fraction)
	    if ~datatype(before, N) | ~datatype(after, N) then
	       do fc = 18
	       signal FormatSyntaxError
	   end
	    if before < length(integer) then do
	       fc = 18
	       signal FormatSyntaxError
	    end
	    if after ~= length(fraction) then do
	       fraction = trunc(('.'fraction'0') + ('.'copies('0', after)'5'), after)
		if integer<1&integer>-1 then integer=integer
	       else integer = integer + (fraction % 1)
	       fraction = substr(fraction, 3)
	    end
	    if fraction >= 0 then
	       return right(integer, before)'.'fraction
	    else
	       return right(integer, before)
	 end

 FormatSyntaxError:
	    if show('F', STDERR) then
	       call writeln(STDERR, '+++ Error' fc 'in line' CallLine':' errortext(fc))
	    else
	       mess='+++ Error' fc 'in line' CallLine':' errortext(fc)
		'Message' mess
	    parse source Func .
	    if Func = 'FUNCTION' then do
		'DEFPUBSCREEN("Workbench")'
	       exit "Err"
		end
	    else do
		'DEFPUBSCREEN("Workbench")'
	       exit 10
		end

PROBT: PROCEDURE

	PARSE ARG TA,K1
	A=.36338023
	W=ATAN(TA/SQRT(K1))
	S=SIN(W)
	C=COS(W)
	L=K1-2*INT(K1/2)
	IF L=0 THEN DO
		T1=S
		IF K1=2 THEN DO
		PO=.5*(1+T1)
		RETURN PO
		END
		T2=S
		J1=-1
		J2=0
		K2=(K1-2)/2
	END
	ELSE DO
		T1=W
		IF K1=1 THEN DO
			T1=T1*(1-A*L)
			PO=.5*(1+T1)
			RETURN PO
		END
		T2=S*C
		T1=T1+T2
		IF K1=3 THEN DO
			T1=T1*(1-A*L)
			PO=.5*(1+T1)
			RETURN PO
		END
		J1=0
		J2=1
		K2=(K1-3)/2
	END
	DO I=1 TO K2
		J1=J1+2
		J2=J2+2
		T2=T2*C*C*J1/J2
		T1=T1+T2
	END
	T1=T1*(1-A*L)
	PO=.5*(1+T1)
RETURN PO

TCRITICAL: PROCEDURE

	PARSE ARG PO,K1
	AO=.0001
	E=.005
	E2=E+E
	A=2*PO-1
	IF K1=1 THEN DO
		TO=TAN(1.57079633*A)
		RETURN TO
	END
	IF K1=2 THEN DO
		SN=SIGN(2/(1-A*A))
		IF SN=-1 THEN TO=-1*(A*SQRT(ABS(2/(1-A*A))))
		ELSE TO=A*SQRT(2/(1-A*A))
		RETURN TO
	END
	P1=PO
	Z=NORMALPP(PO)
	W=Z*(1+2/(1+8*K1))
	T3=K1*(EXP(W*W/K1)-1)
	SELECT
		WHEN Z<0 THEN TT=-1
		WHEN Z=0 THEN TT=0
	OTHERWISE TT=1
	END
	SN=SIGN(T3)
	IF SN=-1 THEN T3=-1*(TT*SQRT(ABS(T3)))
	ELSE T3=TT*SQRT(T3)
	/* LABELA:
	TO=T3
	PO=PROBT(TO,K1)
	F1=PO-P1
	TO=T3+E
	PO=PROBT(TO,K1)
	F2=PO
	TO=T3-E
	PO=PROBT(TO,K1)
	F2=F2-PO
	F2=F2/E2
	T4=T3-F1/F2
	IF ABS(T4-T3)>AO THEN DO
		T3=T4
		SIGNAL 'LABELA'
	END
	*/
	T4=0
	DO FOREVER
		TO=T3
		PO=PROBT(TO,K1)
		F1=PO-P1
		TO=T3+E
		PO=PROBT(TO,K1)
		F2=PO
		TO=T3-E
		PO=PROBT(TO,K1)
		F2=F2-PO
		F2=F2/E2
		T4=T3-F1/F2
		IF ABS(T4-T3)>AO THEN T3=T4
		ELSE LEAVE
	END
	TO=T4
	
RETURN TO

LOGGAMMA: PROCEDURE

	ARG XA
	C1=76.18009173
	C2=-86.50532033
	C3=24.01409822
	C4=-1.231739516
	C5=.001208580
	C6=-.000005364
	C7=2.506628275
	X1=XA-1
	W=X1+5.5
	W=(X1+.5)*LN(W)-W
	S=1+C1/(X1+1)+C2/(X1+2)+C3/(X1+3)
	S=S+C4/(X1+4)+C5/(X1+5)+C6/(X1+6)
	L=W+LN(C7*S)
RETURN L

NORMALPP: PROCEDURE

	ARG P0
	A1=2.515517
	A2=.802853
	A3=.010328
	A4=1.432788
	A5=.189269
	A6=.001308
	Q=.5-ABS(P0-.5)
	SN=SIGN(-2*LN(Q))
	IF SN=-1 THEN W=-1*SQRT(ABS(-2*LN(Q))
	ELSE W=SQRT(-2*LN(Q))
	W1=A1+W*(A2+A3*W)
	W2=1+W*(A4+W*(A5+A6*W))
	ZZ=W-W1/W2
	SELECT
		WHEN (P0-.5)<0 THEN TT=-1
		WHEN (P0-.5)=0 THEN TT=0
		otherwise TT=1
	END
	ZZ=ZZ*TT
RETURN ZZ

NPROB: Procedure Expose PZ Q PDF

	arg Z
	A0=0.5
	A1=0.398942280444
	A2=0.399903438504
	A3=5.75885480458
	A4=29.8213557808
	A5=2.62433121679
	A6=48.6959930692
	A7=5.92885724438
	B0=0.398942280385
	B1=3.8052E-8
	B2=1.00000615302
	B3=3.98064794E-4
	B4=1.98615381364
	B5=0.151679116635
	B6=5.29330324926
	B7=4.8385912808
	B8=15.1508972451
	B9=0.742380924027
	B10=30.789933034
	B11=3.99019417011
	ZABS = ABS(Z)
	Y = A0*Z*Z
	PDF = EXP(-Y)*B0
SELECT
	WHEN (Z>=-1.28) & (Z<=1.28) then DO /*Z BETWEEN -1.28 AND +1.28*/
		Q = A0-ZABS*(A1-A2*Y/(Y+A3-A4/(Y+A5+A6/(Y+A7))))
		IF (Z<0) then do
			PZ = Q
			Q = 1.0-PZ
		End
		Else PZ = 1.0-Q
		RETURN
	End
	WHEN (Z>1.28) & (Z<=12.7) then do /*ZABS BETWEEN 1.28 AND 12.7 */
		Q = PDF/(ZABS-B1+B2/(ZABS+B3+B4/(ZABS-B5+B6/(ZABS+B7-B8/(ZABS+B9+B10/(ZABS+B11))))))
		IF(Z<0) then Do
			PZ = Q
			Q = 1.0-PZ
		End
		Else PZ = 1.0-Q
		RETURN
	End
	WHEN (Z<-1.28) & (Z>=-12.7) then do /*ZABS BETWEEN 1.28 AND 12.7 */
		Q = PDF/(ZABS-B1+B2/(ZABS+B3+B4/(ZABS-B5+B6/(ZABS+B7-B8/(ZABS+B9+B10/(ZABS+B11))))))
		IF(Z<0) then Do
			PZ = Q
			Q = 1.0-PZ
		End
		Else PZ = 1.0-Q
		RETURN
	End
	OTHERWISE /*Z FAR OUT IN TAIL*/
		Q = 0
		PDF = 0
		IF(Z<0) then do
			PZ = Q
			Q = 1.0-PZ
		End
		Else PZ = 1.0
		RETURN
End
Return