/* Friedman 2-Way ANOVA by Ranks */

options results
if ~show('P','TCALC') then do
	address command 'run turbocalc:turbocalc'
	address command 'waitforport TCALC'
	loadflag=1
end
address 'TCALC'
'DEFPUBSCREEN()'
/* Add-in Rexx Math Library needed for some routines */
signal on syntax
if ~show('l','rexxmathlib.library') then
   call addlib('rexxmathlib.library',0,-30)
if ~show('l','rexxreqtools.library') then
   call addlib('rexxreqtools.library',0,-30)
if ~show('l','rexxsupport.library') then
   call addlib('rexxsupport.library',0,-30)
  /* add to library list */
signal off syntax

/* Start Main Routine */
if loadflag=1 then 'Load()'
'ActivateWindow()'
range=rtgetstring(,"Enter Cell Range for Input","Input Request") /*,,'rt_pubscrname="TCALC"')*/
colon=pos(":",range)
if colon=0 then do
	'Message "Please select a range before executing this script"'
	'DEFPUBSCREEN("Workbench")'
	exit
end

/* Find cell references and cell, column numbers */
start_cell=substr(range,1,colon-1)
end_cell=substr(range,colon+1)
start_row=cellrow(start_cell)
end_row=cellrow(end_cell)
start_col=cellcol(start_cell)
end_col=cellcol(end_cell)
NRows=end_row-start_row+1
NCols=end_col-start_col+1

/* Get cell reference for output range */
out_cell=rtgetstring(,"Enter Cell Reference for Output","Input Request") /*,,'rt_pubscrname="TCALC"')*/
if out_cell="" then do
	'DEFPUBSCREEN("Workbench")'
	exit
end
if length(out_cell)<2 | datatype(left(out_cell,1),'n') then do
	'Message "Invalid cell reference"'
	'DEFPUBSCREEN("Workbench")'
	exit
end
/* Suppress Screen Redraw to Speed Things Up */
'Refresh 0'

/* Open a small output window on tcalc screen*/
fo=0
CR='0a'x
DisplayMsg="Calculating...Please Wait."||CR||"User input is disabled during calculation."||CR
if open(6Info, 'con:100/0/450/80/Progress/SCREEN TCALC', w) then do
	 call writeln(6Info, DisplayMsg)
	fo=1
end
else do
	'Message "TCALC Screen not available for Progress messages"'
end
CALL DELAY(150)

/* Get cell references for top cell in each column */
'SelectCell' start_cell
do col=start_col to end_col
	'GetCursorPos'
	top_cell.col=result
	'Column 1'
end

/* Get labels for later use on output */
'SelectCell' start_cell
'GetValue'
testlabel=result
testlabel=strip(testlabel)
if datatype(testlabel,'n')=1 then do
	labelflag=0
	do x=1 to NCols
	title.x="Column "||x
	end
end
else do
	labelflag=1
	NRows=NRows-1
	do x=1 to NCols
	'GetValue'
	str=result
	title.x=translate(strip(str),"_"," ")
	'Column 1'
	end
end
if fo then call writech(6Info,"Progress...10 ")

/* Get data from cell range */
col=start_col
lav=0
tot=0
count.=0
total.=0
do x=1 to NCols
	'SelectCell' top_cell.col
	if labelflag=1 then 'CursorDown 1'
	do y=1 to NRows
		'GetValue'
		valtest=result
		if datatype(valtest)='NUM' then do
			'GetValue'
			val=result
			data.x.y=val
			count.x=1+count.x
		end
		'CursorDown 1'
	end
	col=col+1
	val=0
end
if fo then call writech(6Info,"20 ")

/* Generate Sorted Rows */
srow.=0
Do y=1 to NRows
	Do x=1 to NCols
		srow.x.y=data.x.y
	end
call Sort()
end
if fo then call writech(6Info,"30 ")

/* Calculate Ranks */

Rank.=0
N=NCols
Do y=1 to NRows
	Do x=1 to NCols
	z=0
		Do z=1 to N
		if data.x.y=srow.z.y then Do
			cnt=1
			beginrank=z
			avrank=z
			endrank=0
			Do until endrank ~=0
				z=z+1
				if (data.x.y) ~= (srow.z.y) then endrank=z-1
				avrank=avrank+z
				cnt=cnt+1
				end
			val=trunc((avrank/cnt)-.5,2)
			Rank.x.y=val
			Leave z
			end
		end
	end
end
if fo then call writech(6Info,"40 ")
val=0

/* Calculate Sums etc. of Column Ranks */

sumr.=0
scrsq=0
Rsq.=0
sr.=0
A2=0
B2=0
Do x=1 to NCols
	Do y=1 to NRows
		sumr.x=(sumr.x)+(Rank.x.y)
	end
scrsq=scrsq+(sumr.x)**2
end
Do x=1 to NCols
	Do y=1 to NRows
		Rsq.x.y=(Rank.x.y)**2
		sr.x=sr.x+Rsq.x.y
	end
A2=A2+sr.x
end
B2=scrsq/NRows
if fo then call writech(6Info,"50 ")

/* Calculate Chi-Square

df=0
temp=0
Chi=0
temp=12/((NRows*NCols)*(NCols+1))
temp=temp*scrsq
Chi=temp-(3*NRows)*(NCols+1)
df=NCols-1*/

/* Calculate T2 Statistic */

T2=(NRows-1)*(B2-(NRows*NCols*((NCols+1)**2)/4))
T2=T2/(A2-B2)
k1=NCols-1
k2=(NRows-1)*(NCols-1)
if fo then call writech(6Info,"60 ")

/* Calculate Chi Probability
P=CHIPROB(Chi,df)
*/
/* Calculate Chi Critical
Chicrit1=CHICRIT(.95,df)
Chicrit2=CHICRIT(.99,DF)
*/
/* Calculate F Probability */

AC=.0001
EC=.005
EC2=EC+EC
PF=PROBABILITY(T2,k1,k2)
PF=1-PF
if fo then call writech(6Info,"70 ")

/* P=.95 OR .99 */
FCRIT1=FCRITICAL(.95,k1,k2)
if fo then call writech(6Info,"80 ")
FCRIT2=FCRITICAL(.99,k1,k2)

if fo then do
	call writeln(6Info,"100 ")
	call writeln(6Info,"Writing output to window...")
end
/* Output */
'SelectCell' out_cell
'Column 1'
'ColumnWidth 15'
'Put "Friedman 2-Way ANOVA by Ranks"'
'CursorDown 1'
'Put "Table of Ranks"'
'CursorDown 1'
Do x=1 to NCols
	title=title.x
	'Alignment 2'
	'Put' title
	'CursorDown 1'
	Do y=1 to NRows
		'Put' rank.x.y
		'CursorDown 1'
	end
	'Put' sumr.x
'CursorUp' NRows+1
'Column 1'
end
'SelectCell' out_cell
'CursorDown' NRows+3
'Put "Sum of Ranks:"'
'CursorDown 1'
/*'Put "Chi-Square:"'
'CursorDown 1'
'Put "d.f. (v):"'
'CursorDown 1'
'Put "P(CHI<=chi):"'
'CursorDown 1'
'Put "Chi-Critical (95%):"'
'CursorDown 1'
'Put "Chi-Critical (99%):"'*/
/* Put F stuff here */
'CursorDown 1'
'Put "Friedman T:"'
'CursorDown 1'
'Put "P(F<=T):"'
'CursorDown 1'
'Put "F Critical (95%):"'
'CursorDown 1'
'Put "F Critical (99%):"'
'CursorUp 3'
'Column 1'
/*'Put' format(Chi,,4)
'CursorDown 1'
'Put' df
'CursorDown 1'
'Put' format(P,,6)
'CursorDown 1'
'Put' format(ChiCrit1,,4)
'CursorDown 1'
'Put' format(ChiCrit2,,4)
'CursorDown 1'*/
'Put' format(T2,,4)
'CursorDown 1'
'Put' format(PF,,6)
'CursorDown 1'
'Put' format(FCRIT1,,4)
'CursorDown 1'
'Put' format(FCRIT2,,4)
'Refresh 1'
'Refresh 2'
/*'Message' "Finished"*/
/*indicate the main script is finished*/
DisplayMsg="Cleaning up ...."||CR||"Exiting"
result=writeln(6Info, DisplayMsg)
if result~=0 then do
	/*Wait 3 seconds*/
	CALL DELAY(150)
	/* close window*/
	result=close(6Info)
end
'DEFPUBSCREEN("Workbench")'
exit

/* Procedures */

cellrow: procedure
do
	parse arg cell
	do charpos=2 to length(cell)
	if datatype(substr(cell,charpos,1),n) then return substr(cell,charpos)
	end
	return 0
end
Return

cellcol: procedure
do
	parse arg cell
	labels="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	cell=upper(cell)
	len=length(cell)
	val=0
do charpos=1 to len
	if datatype(substr(cell,charpos,1),n) then
	do cell=reverse(substr(cell,1,charpos-1))
	do x=1 to length(cell)
	val=(26**(x-1))*pos(substr(cell,x,1),labels)+val
	end
	return val
	end
	end
	return 0
end
Return

/* It is important to put the exposed array at the end of the next line */
Sort: procedure expose y NCols srow.
p=y
L=(xtoy(2,int(log(NCols)/log(2))))-1
	Do Until L<1
	L=trunc(int(L/2))
	Do J=1 to L
			Do K=J+L To NCols By L
			I=K
			dumdat=srow.I.p
			Do while I>L
				x=I-L
				If srow.x.p ~> dumdat then Leave
				srow.I.p=srow.x.p
				I=I-L
			End
			srow.I.p=dumdat
			End
		End
	End
Return


syntax:
	 if arg(1)='FAIL' then do
	    'Message "Library is unavailable."'
		'DEFPUBSCREEN("Workbench")'
	    exit
	    end
	'DEFPUBSCREEN("Workbench")'
	exit

CHIPROB: PROCEDURE

	PARSE ARG X2,K1
	X3=0.5*X2
	K3=0.5*K1
	X=K3+1
	LG=LOGGAMMA(X)
	S=0
	A=EXP(K3*LN(X3)-LG-X3)
	IF A~=0 THEN DO
		T=1
		S=1
		G=K3
		DO WHILE (T/S) >.000001
			G=G+1
			T=T*X3/G
			S=S+T
		END
	END
	PO=S*A
RETURN PO

CHICRIT: PROCEDURE

	PARSE ARG PO,K1
	AO=.0001
	E=.005
	E2=E+E
	PA=PO
	SELECT
	WHEN K1=1 THEN DO
		PO=.5+PA/2
		Z=NORMALPP(PO)
		X1=Z*Z
	END
	WHEN K1=2 THEN DO
		X1=1-PA
		X1=-2*LN(X1)
	END
	OTHERWISE
		Z=NORMALPP(PO)
		A1=2/(9*K1)
		W=1-A1+Z*SQRT(A1)
		XO=K1*(W**3)
		DO FOREVER
			X2=XO
			PO=CHIPROB(X2,K1)
			F1=PO-PA
			X2=XO+E
			PO=CHIPROB(X2,K1)
			F2=PO
			X2=XO-E
			PO=CHIPROB(X2,K1)
			F2=F2-PO
			F2=F2/E2
			X1=XO-F1/F2
			IF ABS(X1-XO)>AO THEN XO=X1
			ELSE LEAVE
		END	
	END
	X2=X1
RETURN X2

LOGGAMMA: PROCEDURE

	ARG XA
	C1=76.18009173
	C2=-86.50532033
	C3=24.01409822
	C4=-1.231739516
	C5=.001208580
	C6=-.000005364
	C7=2.506628275
	X1=XA-1
	W=X1+5.5
	W=(X1+.5)*LN(W)-W
	S=1+C1/(X1+1)+C2/(X1+2)+C3/(X1+3)
	S=S+C4/(X1+4)+C5/(X1+5)+C6/(X1+6)
	L=W+LN(C7*S)
RETURN L

NORMALPP: PROCEDURE

	ARG P0
	A1=2.515517
	A2=.802853
	A3=.010328
	A4=1.432788
	A5=.189269
	A6=.001308
	Q=.5-ABS(P0-.5)
	SN=SIGN(-2*LN(Q))
	IF SN=-1 THEN W=-1*SQRT(ABS(-2*LN(Q))
	ELSE W=SQRT(-2*LN(Q))
	W1=A1+W*(A2+A3*W)
	W2=1+W*(A4+W*(A5+A6*W))
	ZZ=W-W1/W2
	SELECT
		WHEN (P0-.5)<0 THEN TT=-1
		WHEN (P0-.5)=0 THEN TT=0
		otherwise TT=1
	END
	ZZ=ZZ*TT
RETURN ZZ

FCRITICAL: PROCEDURE EXPOSE AC EC EC2

	PARSE ARG PO,K1,K2
	/* fIND NORMAL Z CORRESPONDING TO P */
	P1=PO
	Z=NORMALPP(PO)
	IF K2<3 THEN DO
		W=K2**.75
		W=Z/W
		Z=Z*(1.1581-W*(.2296+W*(.0042+.0027*W)))
	END
	/* FIND INITIAL APPROX. TO F */
	A1=2/(9*K1)
	A2=2/(9*K2)
	W=Z*Z
	W1=1+A2*(A2-W-2)
	W2=A1+A2-A1*A2-1
	W3=1+A1*(A1-W-2)
	SN=SIGN(W2*W2-W1*W3)
	IF SN=-1 THEN W3=-1*SQRT(ABS(W2*W2-W1*W3))
	ELSE W3=SQRT(W2*W2-W1*W3)
	FO=(W3-W2)/W1
	FO=FO**3
	/* MODIFIED NEWTON ITERATION TO IMPROVE VALUE OF F */
	DO FOREVER
		FCRIT=FO
		PO=PROBABILITY(FCRIT,K1,K2)
		F1=PO-P1
		FCRIT=FO+EC
		PO=PROBABILITY(FCRIT,K1,K2)
		F2=PO
		FCRIT=FO-EC
		PO=PROBABILITY(FCRIT,K1,K2)
		F2=F2-PO
		F2=F2/EC2
		F1=FO-F1/F2
		IF ABS(F1-FO)>AC THEN FO=F1
		ELSE LEAVE
	END
	FCRIT=F1
RETURN FCRIT

PROBABILITY: PROCEDURE EXPOSE AC EC EC2

	PARSE ARG F,K1,K2
	IF K1=1 THEN AC=.00001
	H2=K2/2
	H1=K1/2
	H3=H1+H2
	L1=0
	XX=H1
	LG=LOGGAMMA(XX)
	L1=L1+LG
	XX=H2
	LG=LOGGAMMA(XX)
	L1=L1+LG
	XX=H3
	LG=LOGGAMMA(XX)
	L1=L1-LG
	W1=K2/(K2+K1*F)
	XX=0
	IF H2<(H3*W1) THEN DO
		W2=W1
		W1=1-W1
		W3=H2
		H2=H1
		H1=W3
	END
	ELSE DO
		W2=1-W1
		XX=1
	END
	T1=1
	W3=1
	P=1
	I=INT(H1+W2*H3)
	W4=W1/W2
	/*LABELB:*/
	T2=H1-W3
	IF I=0 THEN W4=W1
	/*LABELC:*/
	DO FOREVER
		T1=T1*T2*W4/(H2+W3)
		P=P+T1
		T2=ABS(T1)
		IF (T2<=AC) & (T2<=AC*P) THEN LEAVE /*SIGNAL 'LABELD'*/
		W3=W3+1
		I=I-1
		IF I>=0 THEN DO /*SIGNAL 'LABELB'*/
			T2=H1-W3
			IF I=0 THEN W4=W1
			ITERATE
		END
		T2=H3
		H3=H3+1
	/*SIGNAL 'LABELC'*/
	END
	/*LABELD:*/
	W1=EXP(H2*LN(W1)+(H1-1)*LN(W2)-L1)
	P=P*W1/H2
	IF XX=0 THEN PO=P
	ELSE PO=1-P
	AC=.001
RETURN PO

Format:  procedure

	 arg number, before, after
	 CallLine = SIGL
	 if ~datatype(CallLine, 'N') then CallLine = '??'

	 /* Make sure we have a number as first (required) argument    */
	 if ~datatype(number, 'N') then do
	    if number = '' then
	       fc = 17     /* Wrong number of arguments 		  */
	    else
	       fc = 47     /* Arithmetic conversion error       	  */
	    signal FormatSyntaxError
	 end
	 num = number + 0
	 if before = '' & after = '' then
	    return num
	 else do
	    parse var num integer '.' fraction
	    if before = '' then before = length(integer)
	    if after = '' then after = length(fraction)
	    if ~datatype(before, N) | ~datatype(after, N) then
	       do fc = 18
	       signal FormatSyntaxError
	   end
	    if before < length(integer) then do
	       fc = 18
	       signal FormatSyntaxError
	    end
	    if after ~= length(fraction) then do
	       fraction = trunc(('.'fraction'0') + ('.'copies('0', after)'5'), after)
		if integer<1&integer>-1 then integer=integer
	       else integer = integer + (fraction % 1)
	       fraction = substr(fraction, 3)
	    end
	    if fraction >= 0 then
	       return right(integer, before)'.'fraction
	    else
	       return right(integer, before)
	 end

 FormatSyntaxError:
	    if show('F', STDERR) then
	       call writeln(STDERR, '+++ Error' fc 'in line' CallLine':' errortext(fc))
	    else
	       mess='+++ Error' fc 'in line' CallLine':' errortext(fc)
		'Message' mess
	    parse source Func .
	    if Func = 'FUNCTION' then do
		'DEFPUBSCREEN("Workbench")'
	       exit "Err"
		end
	    else do
		'DEFPUBSCREEN("Workbench")'
	       exit 10
		end
