/* Shannon-Wiener Index of Bio-diversity */

options results
if ~show('P','TCALC') then do
	address command 'run turbocalc:turbocalc'
	address command 'waitforport TCALC'
	loadflag=1
end
address 'TCALC'
'DEFPUBSCREEN()'
/* Add-in Rexx Math Library needed for some routines */
signal on syntax
if ~show('l','rexxmathlib.library') then
   call addlib('rexxmathlib.library',0,-30)
if ~show('l','rexxreqtools.library') then
   call addlib('rexxreqtools.library',0,-30)
if ~show('l','rexxsupport.library') then
   call addlib('rexxsupport.library',0,-30)
  /* add to library list */
signal off syntax

/* Start Main Routine */
if loadflag=1 then 'Load()'
'ActivateWindow()'
range=rtgetstring(,"Enter Cell Range for Input","Input Request") /*,,'rt_pubscrname="TCALC"')*/
colon=pos(":",range)
if colon=0 then do
	'Message "Please select a range before executing this script"'
	'DEFPUBSCREEN("Workbench")'
	exit
end

/* Find cell references and cell, column numbers */
start_cell=substr(range,1,colon-1)
end_cell=substr(range,colon+1)
start_row=cellrow(start_cell)
end_row=cellrow(end_cell)
start_col=cellcol(start_cell)
end_col=cellcol(end_cell)
NRows=end_row-start_row+1
NCols=end_col-start_col+1
if (NCols >2) then do
	'Message "Only Two Columns of Data Allowed"'
	'DEFPUBSCREEN("Workbench")'
	exit
end


/* Get cell reference for output range */
out_cell=rtgetstring(,"Enter Cell Reference for Output","Input Request") /*,,'rt_pubscrname="TCALC"')*/
if out_cell="" then do
	'DEFPUBSCREEN("Workbench")'
	exit
end
if length(out_cell)<2 | datatype(left(out_cell,1),'n') then do
	'Message "Invalid cell reference"'
	'DEFPUBSCREEN("Workbench")'
	exit
end
/* Suppress Screen Redraw to Speed Things Up */
'Refresh 0'

/* Open a small output window on tcalc screen*/
fo=0
CR='0a'x
DisplayMsg="Calculating...Please Wait."||CR||"User input is disabled during calculation."||CR
if open(6Info, 'con:100/0/450/80/Progress/SCREEN TCALC', w) then do
	 call writeln(6Info, DisplayMsg)
	fo=1
end
else do
	'Message "TCALC Screen not available for Progress messages"'
end
CALL DELAY(150)

/* Get cell references for top cell in each column */
'SelectCell' start_cell
do col=start_col to end_col
	'GetCursorPos'
	top_cell.col=result
	'Column 1'
end

/* Get labels for later use on output */
title.=""
'SelectCell' start_cell
'GetValue'
testlabel=result
testlabel=strip(testlabel)
if datatype(testlabel,'n')=1 then do
	labelflag=0
	do x=1 to NCols
	title.x="Column "||x
	end
end
else do
	labelflag=1
	NRows=NRows-1
	do x=1 to NCols
	'GetValue'
	str=result
	title.x=translate(strip(str),"_"," ")
	'Column 1'
	end
end

/* Get data from cell range */
col=start_col
lav=0
tot=0
count.=0
total.=0
do x=1 to NCols
	'SelectCell' top_cell.col
	if labelflag=1 then 'CursorDown 1'
	do y=1 to NRows
		'GetValue'
		valtest=result
		if datatype(valtest)='NUM' then do
			'GetValue'
			val=result
			data.x.y=val
			tot=tot+val
			total.x=tot
			count.x=1+count.x
		end
		'CursorDown 1'
	end
	col=col+1
	tot=0
	lav=0
	val=0
end

if fo then call writech(6Info,"Progress...10 ")

/* Calculate log10 of fi */
/* fi is data.x.y */
logfi.=0
logfisq.=0
sumlogfi.=0
sumlogfisq.=0
Do x=1 to NCols
	Do y = 1 to NRows
		dummy=data.x.y
		lg=log10(dummy)
		lgg=pow(lg,2)
		logfi.x.y=lg*dummy
		sumlogfi.x=(sumlogfi.x)+logfi.x.y
		logfisq.x.y=dummy*lgg
		sumlogfisq.x=(sumlogfisq.x)+logfisq.x.y
	end
end
sumfi.=0
Do x=1 to NCols
	Do y = 1 to NRows
		sumfi.x=(sumfi.x)+(data.x.y)
	end
end

if fo then call writech(6Info,"20 ")
/* Calculate Index */
H.=0
TT.=0
Do x=1 to NCols
	TT.x=((sumfi.x)*log10(sumfi.x))-(sumlogfi.x)
	H.x=(TT.x)/(sumfi.x)
end
if fo then call writech(6Info,"30 ")
/* Calculate variance */
tt2.=0
var.=0
Do x=1 to NCols
	tt2.x=((sumlogfi.x)**2)/(sumfi.x)
	var.x=((sumlogfisq.x)-(tt2.x))/((sumfi.x)**2)
end
if fo then call writech(6Info,"40 ")
if NCols>1 then do
/* Calculate StdDev, t and df */
	StDev=0
	StDev=SQRT((var.1)+(var.2))
	t=0
	t=((H.1)-(H.2))/StDev
	df=0
	torp=0
	torp=(((var.1)**2)/(total.1))+(((var.2)**2/(total.2)))
	df=(((var.1)+(var.2))**2)/torp
if fo then call writech(6Info,"50 ")
/* Calculate T distribution function */
	dfi=df
	dfi=int(dfi)+0
	P=PROBT(t,dfi)
	if t>=0 then P=1-P
	PT=P*2
if fo then call writech(6Info,"60 ")
/* Calculate T Critical */

	TCRIT1=TCRITICAL(.95,dfi)
	if fo then call writech(6Info,"70 ")
	TCRIT2=TCRITICAL(.99,dfi)
	if fo then call writech(6Info,"80 ")
	TCRIT3=TCRITICAL(.975,dfi)
	if fo then call writech(6Info,"90 ")
	TCRIT4=TCRITICAL(.995,dfi)
end
if fo then do
	call writeln(6Info,"100")
	call writeln(6Info,"Writing output to window...")
end

/* output */
'SelectCell' out_cell
'ColumnWidth 12' 
'Put "Shannon-Wiener Index of Bio-diversity"'
'CursorDown 1'
'Put "Using Log10 as Base"'
'CursorDown 2'
'GetCursorPos'
top_cell=result
'CursorDown 1'
'Put "Total Count"'
'CursorDown 1'
'Put "No. Classes"'
'CursorDown 2'
'Put "Index (H)"'
'CursorDown 2'
'Put "Variance"'
'GetCursorPos'
vr_cell=result
Do x=1 to NCols
	'SelectCell' top_cell
	'Column' x
	'Alignment 2'
	'Put' title.x
	'CursorDown 1'
	'Put' total.x
	'CursorDown 1'
	'Put' count.x
	'CursorDown 2'
	'Put' format(H.x,,4)
	'CursorDown 2'
	'Put' format(var.x,,5)
end
if NCols>1 then do
	'SelectCell' vr_cell
	'CursorDown 2'
	'Put "Std. Dev.:"'
	'CursorDown 1'
	'Put "t:"'
	'CursorDown 1'
	'Put "d.f. (v):"'
	'CursorDown 1'
	'Put "P(T<=t) one-tail:"'
	'CursorDown 1'
	'Put "T-Critical (95%):"'
	'CursorDown 1'
	'Put "T-Critical (99%):"'
	'CursorDown 1'
	'Put "P(T<=t) two-tail:"'
	'CursorDown 1'
	'Put "T-Critical (97.5%):"'
	'CursorDown 1'
	'Put "T-Critical (99.5%):"'
	'SelectCell' vr_cell
	'Column 1'
	'CursorDown 2'
	'Put' format(StDev,,4)
	'CursorDown 1'
	'Put' format(t,,4)
	'CursorDown 1'
	'Put' format(df,,4)
	'CursorDown 1'
	'Put' format(P,,6)
	'CursorDown 1'
	'Put' format(TCRIT1,,4)
	'CursorDown 1'
	'Put' format(TCRIT2,,4)
	'CursorDown 1'
	'Put' format(PT,,6)
	'CursorDown 1'
	'Put' format(TCRIT3,,4)
	'CursorDown 1'
	'Put' format(TCRIT4,,4)
end
'Refresh 1'
'Refresh 2'
/*'Message "Finished"'*/
/*indicate the main script is finished*/
DisplayMsg="Cleaning up ...."||CR||"Exiting"
result=writeln(6Info, DisplayMsg)
if result~=0 then do
	/*Wait 3 seconds*/
	CALL DELAY(150)
	/* close window*/
	result=close(6Info)
end
'DEFPUBSCREEN("Workbench")'
exit

/* Procedures */

cellrow: procedure
do
	parse arg cell
	do charpos=2 to length(cell)
	if datatype(substr(cell,charpos,1),n) then return substr(cell,charpos)
	end
	return 0
end
Return

cellcol: procedure
do
	parse arg cell
	labels="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	cell=upper(cell)
	len=length(cell)
	val=0
do charpos=1 to len
	if datatype(substr(cell,charpos,1),n) then
	do cell=reverse(substr(cell,1,charpos-1))
	do x=1 to length(cell)
	val=(26**(x-1))*pos(substr(cell,x,1),labels)+val
	end
	return val
	end
	end
	return 0
end
Return

syntax:
	 if arg(1)='FAIL' then do
	    'Message "Library is unavailable."'
		'DEFPUBSCREEN("Workbench")'
	    exit
	    end
	'DEFPUBSCREEN("Workbench")'
	exit

Format:  procedure

	 arg number, before, after
	 CallLine = SIGL
	 if ~datatype(CallLine, 'N') then CallLine = '??'

	 /* Make sure we have a number as first (required) argument    */
	 if ~datatype(number, 'N') then do
	    if number = '' then
	       fc = 17     /* Wrong number of arguments 		  */
	    else
	       fc = 47     /* Arithmetic conversion error       	  */
	    signal FormatSyntaxError
	 end
	 num = number + 0
	 if before = '' & after = '' then
	    return num
	 else do
	    parse var num integer '.' fraction
	    if before = '' then before = length(integer)
	    if after = '' then after = length(fraction)
	    if ~datatype(before, N) | ~datatype(after, N) then
	       do fc = 18
	       signal FormatSyntaxError
	   end
	    if before < length(integer) then do
	       fc = 18
	       signal FormatSyntaxError
	    end
	    if after ~= length(fraction) then do
	       fraction = trunc(('.'fraction'0') + ('.'copies('0', after)'5'), after)
		if integer<1&integer>-1 then integer=integer
	       else integer = integer + (fraction % 1)
	       fraction = substr(fraction, 3)
	    end
	    if fraction >= 0 then
	       return right(integer, before)'.'fraction
	    else
	       return right(integer, before)
	 end

 FormatSyntaxError:
	    if show('F', STDERR) then
	       call writeln(STDERR, '+++ Error' fc 'in line' CallLine':' errortext(fc))
	    else
	       mess='+++ Error' fc 'in line' CallLine':' errortext(fc)
		'Message' mess
	    parse source Func .
	    if Func = 'FUNCTION' then do
		'DEFPUBSCREEN("Workbench")'
	       exit "Err"
		end
	    else do
		'DEFPUBSCREEN("Workbench")'
	       exit 10
		end

PROBT: PROCEDURE

	PARSE ARG TA,K1
	A=.36338023
	W=ATAN(TA/SQRT(K1))
	S=SIN(W)
	C=COS(W)
	L=K1-2*INT(K1/2)
	IF L=0 THEN DO
		T1=S
		IF K1=2 THEN DO
			PO=.5*(1+T1)
			RETURN PO
		END
		T2=S
		J1=-1
		J2=0
		K2=(K1-2)/2
	END
	ELSE DO
		T1=W
		IF K1=1 THEN DO
			T1=T1*(1-A*L)
			PO=.5*(1+T1)
			RETURN PO
		END
		T2=S*C
		T1=T1+T2
		IF K1=3 THEN DO
			T1=T1*(1-A*L)
			PO=.5*(1+T1)
			RETURN PO
		END
		J1=0
		J2=1
		K2=(K1-3)/2
	END
	DO I=1 TO K2
		J1=J1+2
		J2=J2+2
		T2=T2*C*C*J1/J2
		T1=T1+T2
	END
	T1=T1*(1-A*L)
	PO=.5*(1+T1)
RETURN PO

TCRITICAL: PROCEDURE

	PARSE ARG PO,K1
	AO=.0001
	E=.005
	E2=E+E
	A=2*PO-1
	IF K1=1 THEN DO
		TO=TAN(1.57079633*A)
		RETURN TO
	END
	IF K1=2 THEN DO
		SN=SIGN(2/(1-A*A))
		IF SN=-1 THEN TO=-1*(A*SQRT(ABS(2/(1-A*A))))
		ELSE TO=A*SQRT(2/(1-A*A))
		RETURN TO
	END
	P1=PO
	Z=NORMALPP(PO)
	W=Z*(1+2/(1+8*K1))
	T3=K1*(EXP(W*W/K1)-1)
	SELECT
		WHEN Z<0 THEN TT=-1
		WHEN Z=0 THEN TT=0
	OTHERWISE TT=1
	END
	SN=SIGN(T3)
	IF SN=-1 THEN T3=-1*(TT*SQRT(ABS(T3)))
	ELSE T3=TT*SQRT(T3)
	T4=0
	DO FOREVER
		TO=T3
		PO=PROBT(TO,K1)
		F1=PO-P1
		TO=T3+E
		PO=PROBT(TO,K1)
		F2=PO
		TO=T3-E
		PO=PROBT(TO,K1)
		F2=F2-PO
		F2=F2/E2
		T4=T3-F1/F2
		IF ABS(T4-T3)>AO THEN T3=T4
		ELSE LEAVE
	END
	TO=T4
	
RETURN TO

LOGGAMMA: PROCEDURE

	ARG XA
	C1=76.18009173
	C2=-86.50532033
	C3=24.01409822
	C4=-1.231739516
	C5=.001208580
	C6=-.000005364
	C7=2.506628275
	X1=XA-1
	W=X1+5.5
	W=(X1+.5)*LN(W)-W
	S=1+C1/(X1+1)+C2/(X1+2)+C3/(X1+3)
	S=S+C4/(X1+4)+C5/(X1+5)+C6/(X1+6)
	L=W+LN(C7*S)
RETURN L

NORMALPP: PROCEDURE

	ARG P0
	A1=2.515517
	A2=.802853
	A3=.010328
	A4=1.432788
	A5=.189269
	A6=.001308
	Q=.5-ABS(P0-.5)
	SN=SIGN(-2*LN(Q))
	IF SN=-1 THEN W=-1*SQRT(ABS(-2*LN(Q))
	ELSE W=SQRT(-2*LN(Q))
	W1=A1+W*(A2+A3*W)
	W2=1+W*(A4+W*(A5+A6*W))
	ZZ=W-W1/W2
	SELECT
		WHEN (P0-.5)<0 THEN TT=-1
		WHEN (P0-.5)=0 THEN TT=0
		otherwise TT=1
	END
	ZZ=ZZ*TT
RETURN ZZ