/* Smironov Test 2 Samples */

options results
if ~show('P','TCALC') then do
	address command 'run turbocalc:turbocalc'
	address command 'waitforport TCALC'
	loadflag=1
end
address 'TCALC'
'DEFPUBSCREEN()'
/* Add-in Rexx Math Library needed for some routines */
signal on syntax
if ~show('l','rexxmathlib.library') then
   call addlib('rexxmathlib.library',0,-30) 
if ~show('l','rexxreqtools.library') then
   call addlib('rexxreqtools.library',0,-30)
if ~show('l','rexxsupport.library') then
   call addlib('rexxsupport.library',0,-30)
 /* add to library list */
signal off syntax

/* Start Main Routine */
if loadflag=1 then 'Load()'
'ActivateWindow()'
range=rtgetstring(,"Enter Cell Range for Input","Input Request") /*,,'rt_pubscrname="TCALC"')*/
colon=pos(":",range)
if colon=0 then do
	'Message "Please select a range before executing this script"'
	'DEFPUBSCREEN("Workbench")'
	exit
end

/* Find cell references and cell, column numbers */
start_cell=substr(range,1,colon-1)
end_cell=substr(range,colon+1)
start_row=cellrow(start_cell)
end_row=cellrow(end_cell)
start_col=cellcol(start_cell)
end_col=cellcol(end_cell)
NRows=end_row-start_row+1
NCols=end_col-start_col+1
if NCols>2 then do
	'Message "Only two columns allowed!"'
	'DEFPUBSCREEN("Workbench")'
	exit
end

/* Get cell reference for output range */
out_cell=rtgetstring(,"Enter Cell Reference for Output","Input Request") /*,,'rt_pubscrname="TCALC"')*/
if out_cell="" then do
	'DEFPUBSCREEN("Workbench")'
	exit
end
if length(out_cell)<2 | datatype(left(out_cell,1),'n')=1 then do
	'Message "Invalid cell reference"'
	'DEFPUBSCREEN("Workbench")'
	exit
end
/* Suppress Screen Redraw to Speed Things Up */
'Refresh 0'

/* Open a small output window on tcalc screen*/
fo=0
CR='0a'x
DisplayMsg="Calculating...Please Wait."||CR||"User input is disabled during calculation."||CR
if open(6Info, 'con:100/0/450/80/Progress/SCREEN TCALC', w) then do
	 call writeln(6Info, DisplayMsg)
	fo=1
end
else do
	'Message "TCALC Screen not available for Progress messages"'
end
CALL DELAY(150)

/* Get cell references for top cell in each column */
'SelectCell' start_cell
do col=start_col to end_col
	'GetCursorPos'
	top_cell.col=result
	'Column 1'
end

/* Get labels for later use on output */
'SelectCell' start_cell
'GetValue'
testlabel=result
testlabel=strip(testlabel)
if datatype(testlabel,'n')=1 then do
	labelflag=0
	do x=1 to NCols
	title.x="Column "||x
	end
end
else do
	labelflag=1
	NRows=NRows-1
	do x=1 to NCols
	'GetValue'
	str=result
	title.x=translate(strip(str),"_"," ")
	'Column 1'
	end
end
if fo then call writech(6Info,"Progress...10 ")

/* Get data from cell range */
col=start_col
lav=0
tot=0
count.=0
total.=0
do x=1 to NCols
	'SelectCell' top_cell.col
	if labelflag=1 then 'CursorDown 1'
	do y=1 to NRows
		'GetValue'
		valtest=result
		if datatype(valtest)='NUM' then do
			'GetValue'
			val=result
			data.x.y=val
			tot=tot+val
			total.x=tot
			count.x=1+count.x
		end
		'CursorDown 1'
	end
	col=col+1
	tot=0
	lav=0
	val=0
end
TCount=count.1 + count.2
if fo then call writech(6Info,"20 ")

/* Sort the Data*/

call Sort()
if fo then call writech(6Info,"30 ")

/*Create combined data array*/

temp.=0
i=0
Do x=1 to NCols
	Do y=1 to count.x
		i=i+1
		temp.i=data.x.y
	end
end
if fo then call writech(6Info,"40 ")

/*Sort combined array*/

Call SortB(TCount)
if fo then call writech(6Info,"50 ")

/*Calculate first empirical distribution function*/

a=0
y=0
s1.=0
Do a=1 to count.1
	Select
		when (data.1.a>temp.y)&(a=1) then DO
			s1.y=0
			a=a-1
			end
		when data.1.a=temp.y then s1.y=a/count.1
		when data.1.a<temp.y then DO
			s1.y=1
			a=a-1
			end
		otherwise
			s1.y=(a-1)/count.1
			a=a-1
	end
	IF a=count.1 then a=a-1
	y=y+1
	IF y>TCount then leave
end
if fo then call writech(6Info,"60 ")

/*Calculate second empirical distribution function*/

a=0
y=0
s2.=0
Do a=1 to count.2
	Select
		when (data.2.a>temp.y)&(a=1) then DO
			s2.y=0
			a=a-1
			end
		when data.2.a=temp.y then s2.y=a/count.2
		when data.2.a<temp.y then DO
			s2.y=1
			a=a-1
			end
		otherwise
			s2.y=(a-1)/count.2
			a=a-1
	end
	IF a=count.2 then a=a-1
	y=y+1
	IF y>TCount then leave
end
if fo then call writech(6Info,"70 ")

/* Calculate Test Statistic*/
Sa.=0
Sb.=0
MaxS1=0
MaxS2=0
DO i=1 to TCount
	Sa.i=S1.i-S2.i
	Sb.i=S2.i-S1.i
	IF (Sa.i)>MaxS1 then MaxS1=Sa.i
	IF (Sb.i)>MaxS2 then MaxS2=Sb.i
	Sd.i=ABS(Sa.i)
end
if fo then call writech(6Info,"80 ")
Call SortC(TCOUNT)
SMIRT=Sd.TCount
if fo then do
	call writeln(6Info,"100")
	call writeln(6Info,"Writing output to window...")
end
/* Output */
'SelectCell' out_cell
'ColumnWidth' 10
'Put "Smirnov Two Sample Test "'
'CursorDown 2'
'GetCursorPos'
top_cell=result
'Alignment 2'
'Put "S1(x)"'
Do y=1 to TCount
	'CursorDown 1'
	'Put' format(S1.y,,4)
end
'SelectCell' top_cell
'Column 1'
'Alignment 2'
'Put "S2(x)"'
Do y=1 to TCount
	'CursorDown 1'
	'Put' format(S2.y,,4)
end
'SelectCell' top_cell
'Column 2'
'Alignment 2'
'ColumnWidth' 13
'Put "S1(x)-S2(x)"'
Do y=1 to TCount
	'CursorDown 1'
	'Put' format(Sa.y,,4)
end
'SelectCell' top_cell
'Column 3'
'Alignment 2'
'ColumnWidth' 13
'Put "S2(x)-S1(x)"'
Do y=1 to TCount
	'CursorDown 1'
	'Put' format(Sb.y,,4)
end
'SelectCell' top_cell
'Column 4'
'Alignment 2'
'ColumnWidth' 12
'Put "Smirnov T"'
'CursorDown 1'
'Alignment 2'
'Put "(Two-Sided)"'
'CursorDown 1'
'Put' format(SMIRT,,6)
'CursorDown 1'
'Put "(One-Sided:S1>S2)"'
'CursorDown 1'
'Put' format(MaxS1,,6)
'CursorDown 1'
'Put "(One-Sided:S2>S1)"'
'CursorDown 1'
'Put' format(MaxS2,,6)
'Refresh 1'
'Refresh 2'
/*'Message' "Finished"*/
/*indicate the main script is finished*/
DisplayMsg="Cleaning up ...."||CR||"Exiting"
result=writeln(6Info, DisplayMsg)
if result~=0 then do
	/*Wait 3 seconds*/
	CALL DELAY(150)
	/* close window*/
	result=close(6Info)
end
'DEFPUBSCREEN("Workbench")'
exit

/* Procedures */

cellrow: procedure
do
	parse arg cell
	do charpos=2 to length(cell)
	if datatype(substr(cell,charpos,1),n) then return substr(cell,charpos)
	end
	return 0
end
Return

cellcol: procedure
do
	parse arg cell
	labels="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	cell=upper(cell)
	len=length(cell)
	val=0
do charpos=1 to len
	if datatype(substr(cell,charpos,1),n) then
	do cell=reverse(substr(cell,1,charpos-1))
	do x=1 to length(cell)
	val=(26**(x-1))*pos(substr(cell,x,1),labels)+val
	end
	return val
	end
	end
	return 0
end
Return

/* It is important to put the exposed array at the end of the next line */
Sort: procedure expose NCols NRows data.

do x=1 to NCols
L=(xtoy(2,int(log(NRows-1)/log(2))))-1
	Do Until L<1
	L=trunc(int(L/2))
	Do J=1 to L
			Do K=J+L To NRows-1 By L
			I=K
			dumdat=data.x.I
			Do while I>L
				y=I-L
				If data.x.y ~> dumdat then Leave
				data.x.I=data.x.y
				I=I-L
			End
			data.x.I=dumdat
			End
		End
	End
End
Return

SortB: procedure expose temp.

arg NRows
L=(xtoy(2,int(log(NRows)/log(2))))-1
	Do Until L<1
	L=trunc(int(L/2))
	Do J=1 to L
			Do K=J+L To NRows By L
			I=K
			dumdat=temp.I
			Do while I>L
				y=I-L
				If temp.y ~> dumdat then Leave
				temp.I=temp.y
				I=I-L
			End
			temp.I=dumdat
			End
		End
	End
Return

SortC: procedure expose Sd.

arg NRows
L=(xtoy(2,int(log(NRows)/log(2))))-1
	Do Until L<1
	L=trunc(int(L/2))
	Do J=1 to L
			Do K=J+L To NRows By L
			I=K
			dumdat=Sd.I
			Do while I>L
				y=I-L
				If Sd.y ~> dumdat then Leave
				Sd.I=Sd.y
				I=I-L
			End
			Sd.I=dumdat
			End
		End
	End
Return

syntax:
	 if arg(1)='FAIL' then do
	    'Message "Library is unavailable."'
		'DEFPUBSCREEN("Workbench")'
	    exit
	    end
	'DEFPUBSCREEN("Workbench")'
	exit

Format:  procedure

	 arg number, before, after
	 CallLine = SIGL
	 if ~datatype(CallLine, 'N') then CallLine = '??'

	 /* Make sure we have a number as first (required) argument    */
	 if ~datatype(number, 'N') then do
	    if number = '' then
	       fc = 17     /* Wrong number of arguments 		  */
	    else
	       fc = 47     /* Arithmetic conversion error       	  */
	    signal FormatSyntaxError
	 end
	 num = number + 0
	 if before = '' & after = '' then
	    return num
	 else do
	    parse var num integer '.' fraction
	    if before = '' then before = length(integer)
	    if after = '' then after = length(fraction)
	    if ~datatype(before, N) | ~datatype(after, N) then
	       do fc = 18
	       signal FormatSyntaxError
	   end
	    if before < length(integer) then do
	       fc = 18
	       signal FormatSyntaxError
	    end
	    if after ~= length(fraction) then do
	       fraction = trunc(('.'fraction'0') + ('.'copies('0', after)'5'), after)
		if integer<1&integer>-1 then integer=integer
	       else integer = integer + (fraction % 1)
	       fraction = substr(fraction, 3)
	    end
	    if fraction >= 0 then
	       return right(integer, before)'.'fraction
	    else
	       return right(integer, before)
	 end

 FormatSyntaxError:
	    if show('F', STDERR) then
	       call writeln(STDERR, '+++ Error' fc 'in line' CallLine':' errortext(fc))
	    else
	       mess='+++ Error' fc 'in line' CallLine':' errortext(fc)
		'Message' mess
	    parse source Func .
	    if Func = 'FUNCTION' then do
		'DEFPUBSCREEN("Workbench")'
	       exit "Err"
		end
	    else do
		'DEFPUBSCREEN("Workbench")'
	       exit 10
		end